#include <gst/gst.h>

GstElement *pipeline, *rtpbin, *asource, *aencoder, *payloader, 
					 *udpsink0, *udpsink1, *udpsource;

static gboolean
bus_call (GstBus *bus, GstMessage *msg, gpointer data)
{
  GMainLoop *loop = (GMainLoop *) data;

  switch (GST_MESSAGE_TYPE (msg)) {
    case GST_MESSAGE_EOS:
      g_print ("End-of-stream\n");
      g_main_loop_quit (loop);
      break;
    case GST_MESSAGE_ERROR: {
      gchar *debug;
      GError *err;

      gst_message_parse_error (msg, &err, &debug);
      g_free (debug);

      g_print ("Error: %s\n", err->message);
      g_error_free (err);

      g_main_loop_quit (loop);
      break;
    }
    default:
      break;
  }

  return TRUE;
}

static void new_pad_send_rtp_src(  )
{
	GstPad *sinkpad;
	GstPad *srcpad;
	gchar *srcname;
	gchar *sinkname;	
  //gboolean b;
	
	sinkpad = gst_element_get_static_pad(udpsink0, "sink");
	sinkname = gst_pad_get_name(sinkpad);
	srcpad = gst_element_get_pad(rtpbin, "send_rtp_src_0");
	srcname = gst_pad_get_name(srcpad);
	/*
	b = gst_pad_is_active(srcpad);
	
	if (b)
		g_print ("active \n");
	else
		g_print ("not active \n");		
	
	b = gst_pad_can_link(srcpad, sinkpad);	
	
	if (b)
		g_print ("can link \n");
	else
		g_print ("can not link \n");
	*/
	gst_pad_link (srcpad, sinkpad);
	g_print ("A dynamic pad %s created and linked \n", srcname);

	g_free(srcname);
	g_free(sinkname);
	gst_object_unref(sinkpad);
	gst_object_unref(srcpad);

}

static void new_pad_recv_rtcp_sink(  )
{
	GstPad *sinkpad;
	GstPad *srcpad;
	gchar *sinkname;
	
	srcpad = gst_element_get_static_pad(udpsource, "src");
	sinkpad = gst_element_get_request_pad(rtpbin, "recv_rtcp_sink_0");
	sinkname = gst_pad_get_name (sinkpad);
	
	gst_pad_link(srcpad, sinkpad);
	g_print("A new sink pad %s was created and linked \n", sinkname);
	
	g_free(sinkname);
		
	gst_object_unref(srcpad);
	gst_object_unref(sinkpad);	
	
}

static void new_pad_send_rtp_sink(  )
{
	GstPad *sinkpad;
	GstPad *srcpad;
	gchar *sinkname;
	
	srcpad = gst_element_get_static_pad(payloader, "src");
	sinkpad = gst_element_get_request_pad(rtpbin, "send_rtp_sink_0");
	sinkname = gst_pad_get_name (sinkpad);
	
	gst_pad_link(srcpad, sinkpad);
	g_print("A new sink pad %s was created and linked \n", sinkname);
	
	g_free(sinkname);
		
	gst_object_unref(srcpad);
	gst_object_unref(sinkpad);	
}

static void new_pad_send_rtcp_src(  )
{
	GstPad *sinkpad;
	GstPad *srcpad;
	gchar *srcname;
	
	sinkpad = gst_element_get_static_pad(udpsink1, "sink");
	srcpad = gst_element_get_request_pad(rtpbin, "send_rtcp_src_0");
	srcname = gst_pad_get_name (srcpad);
	
	gst_pad_link(srcpad, sinkpad);
	g_print("A new source pad %s was created and linked \n", srcname);
	
	g_free(srcname);
		
	gst_object_unref(srcpad);
	gst_object_unref(sinkpad);	
}

int main (int argc, char *argv[])
{
  GMainLoop *loop;
  GstBus *bus;

  /* initialize GStreamer */
  gst_init (&argc, &argv);
  loop = g_main_loop_new (NULL, FALSE);

  /* create elements */
  pipeline = gst_pipeline_new ("rtp-audio");
  rtpbin = gst_element_factory_make ("gstrtpbin", "rtp-handler");
  asource = gst_element_factory_make ("audiotestsrc", "audio-source");
  aencoder = gst_element_factory_make ("alawenc", "law-encoder");
  payloader = gst_element_factory_make ("rtppcmapay", "pcma-payloader");
  udpsink0 = gst_element_factory_make ("udpsink", "udp-sink0");
  udpsink1 = gst_element_factory_make ("udpsink", "udp-sink1");
  udpsource = gst_element_factory_make ("udpsrc", "udp-src");
  
  if (!pipeline || !asource ||!aencoder || !payloader || !udpsink0 || !udpsink1 || !udpsource) {
    g_print ("One element could not be created\n");
    return -1;
  }

  /* set filename property on the file source. Also add a message
   * handler. */
  g_object_set (G_OBJECT (udpsink0), "port", 5000, NULL);
  g_object_set (G_OBJECT (udpsink1), "port", 5001, NULL);
  g_object_set (G_OBJECT (udpsource), "port", 5005, NULL);
/*  
  g_object_set (G_OBJECT (udpsink0), "sock", 6, NULL);
  g_object_set (G_OBJECT (udpsink1), "sock", 5, NULL);
  g_object_set (G_OBJECT (udpsource), "sock", 9, NULL);
*/
  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
  gst_bus_add_watch (bus, bus_call, loop);
  gst_object_unref (bus);

  /* put all elements in a bin */
  gst_bin_add_many (GST_BIN (pipeline), 
  									asource, aencoder, payloader, udpsink0, udpsink1, udpsource, NULL);

  /* link together - note that we cannot link rtpbin to the other
   * elements yet, because it uses request and dynamic pads. For that,
   * we set some pad-adder functions. */
  gst_element_link_many(asource, aencoder, payloader, NULL);
  
  //g_signal_connect(rtpbin, "pad-added", G_CALLBACK(new_pad), NULL);
	new_pad_recv_rtcp_sink(  ); // req pad
	new_pad_send_rtp_sink(  );  // req pad
	new_pad_send_rtcp_src(  );  // req pad
	new_pad_send_rtp_src(  );   // dyn pad

  /* Now set to playing and iterate. */
  g_print ("Setting to PLAYING\n");
  gst_element_set_state (pipeline, GST_STATE_PLAYING);
  g_print ("Running\n");
  g_main_loop_run (loop);

  /* clean up nicely */
  g_print ("Returned, stopping playback\n");
  gst_element_set_state (pipeline, GST_STATE_NULL);
  g_print ("Deleting pipeline\n");
  gst_object_unref (GST_OBJECT (pipeline));

  return 0;
}
